# encoding: utf-8
# Code generated by Microsoft (R) AutoRest Code Generator.
# Changes may cause incorrect behavior and will be lost if the code is
# regenerated.

module Azure::EventGrid::Mgmt::V2020_04_01_preview
  #
  # Azure EventGrid Management Client
  #
  class PrivateLinkResources
    include MsRestAzure

    #
    # Creates and initializes a new instance of the PrivateLinkResources class.
    # @param client service class for accessing basic functionality.
    #
    def initialize(client)
      @client = client
    end

    # @return [EventGridManagementClient] reference to the EventGridManagementClient
    attr_reader :client

    #
    # Get a private link resource.
    #
    # Get properties of a private link resource.
    #
    # @param resource_group_name [String] The name of the resource group within the
    # user's subscription.
    # @param parent_type [String] The type of the parent resource. This can be
    # either \'topics\' or \'domains\'.
    # @param parent_name [String] The name of the parent resource (namely, either,
    # the topic name or domain name).
    # @param private_link_resource_name [String] The name of private link resource.
    # @param custom_headers [Hash{String => String}] A hash of custom headers that
    # will be added to the HTTP request.
    #
    # @return [PrivateLinkResource] operation results.
    #
    def get(resource_group_name, parent_type, parent_name, private_link_resource_name, custom_headers:nil)
      response = get_async(resource_group_name, parent_type, parent_name, private_link_resource_name, custom_headers:custom_headers).value!
      response.body unless response.nil?
    end

    #
    # Get a private link resource.
    #
    # Get properties of a private link resource.
    #
    # @param resource_group_name [String] The name of the resource group within the
    # user's subscription.
    # @param parent_type [String] The type of the parent resource. This can be
    # either \'topics\' or \'domains\'.
    # @param parent_name [String] The name of the parent resource (namely, either,
    # the topic name or domain name).
    # @param private_link_resource_name [String] The name of private link resource.
    # @param custom_headers [Hash{String => String}] A hash of custom headers that
    # will be added to the HTTP request.
    #
    # @return [MsRestAzure::AzureOperationResponse] HTTP response information.
    #
    def get_with_http_info(resource_group_name, parent_type, parent_name, private_link_resource_name, custom_headers:nil)
      get_async(resource_group_name, parent_type, parent_name, private_link_resource_name, custom_headers:custom_headers).value!
    end

    #
    # Get a private link resource.
    #
    # Get properties of a private link resource.
    #
    # @param resource_group_name [String] The name of the resource group within the
    # user's subscription.
    # @param parent_type [String] The type of the parent resource. This can be
    # either \'topics\' or \'domains\'.
    # @param parent_name [String] The name of the parent resource (namely, either,
    # the topic name or domain name).
    # @param private_link_resource_name [String] The name of private link resource.
    # @param [Hash{String => String}] A hash of custom headers that will be added
    # to the HTTP request.
    #
    # @return [Concurrent::Promise] Promise object which holds the HTTP response.
    #
    def get_async(resource_group_name, parent_type, parent_name, private_link_resource_name, custom_headers:nil)
      fail ArgumentError, '@client.subscription_id is nil' if @client.subscription_id.nil?
      fail ArgumentError, 'resource_group_name is nil' if resource_group_name.nil?
      fail ArgumentError, 'parent_type is nil' if parent_type.nil?
      fail ArgumentError, 'parent_name is nil' if parent_name.nil?
      fail ArgumentError, 'private_link_resource_name is nil' if private_link_resource_name.nil?
      fail ArgumentError, '@client.api_version is nil' if @client.api_version.nil?


      request_headers = {}
      request_headers['Content-Type'] = 'application/json; charset=utf-8'

      # Set Headers
      request_headers['x-ms-client-request-id'] = SecureRandom.uuid
      request_headers['accept-language'] = @client.accept_language unless @client.accept_language.nil?
      path_template = 'subscriptions/{subscriptionId}/resourceGroups/{resourceGroupName}/providers/Microsoft.EventGrid/{parentType}/{parentName}/privateLinkResources/{privateLinkResourceName}'

      request_url = @base_url || @client.base_url

      options = {
          middlewares: [[MsRest::RetryPolicyMiddleware, times: 3, retry: 0.02], [:cookie_jar]],
          path_params: {'subscriptionId' => @client.subscription_id,'resourceGroupName' => resource_group_name,'parentType' => parent_type,'parentName' => parent_name,'privateLinkResourceName' => private_link_resource_name},
          query_params: {'api-version' => @client.api_version},
          headers: request_headers.merge(custom_headers || {}),
          base_url: request_url
      }
      promise = @client.make_request_async(:get, path_template, options)

      promise = promise.then do |result|
        http_response = result.response
        status_code = http_response.status
        response_content = http_response.body
        unless status_code == 200
          error_model = JSON.load(response_content)
          fail MsRestAzure::AzureOperationError.new(result.request, http_response, error_model)
        end

        result.request_id = http_response['x-ms-request-id'] unless http_response['x-ms-request-id'].nil?
        result.correlation_request_id = http_response['x-ms-correlation-request-id'] unless http_response['x-ms-correlation-request-id'].nil?
        result.client_request_id = http_response['x-ms-client-request-id'] unless http_response['x-ms-client-request-id'].nil?
        # Deserialize Response
        if status_code == 200
          begin
            parsed_response = response_content.to_s.empty? ? nil : JSON.load(response_content)
            result_mapper = Azure::EventGrid::Mgmt::V2020_04_01_preview::Models::PrivateLinkResource.mapper()
            result.body = @client.deserialize(result_mapper, parsed_response)
          rescue Exception => e
            fail MsRest::DeserializationError.new('Error occurred in deserializing the response', e.message, e.backtrace, result)
          end
        end

        result
      end

      promise.execute
    end

    #
    # List private link resources under specific topic or domain.
    #
    # List all the private link resources under a topic or domain.
    #
    # @param resource_group_name [String] The name of the resource group within the
    # user's subscription.
    # @param parent_type [String] The type of the parent resource. This can be
    # either \'topics\' or \'domains\'.
    # @param parent_name [String] The name of the parent resource (namely, either,
    # the topic name or domain name).
    # @param filter [String] The query used to filter the search results using
    # OData syntax. Filtering is permitted on the 'name' property only and with
    # limited number of OData operations. These operations are: the 'contains'
    # function as well as the following logical operations: not, and, or, eq (for
    # equal), and ne (for not equal). No arithmetic operations are supported. The
    # following is a valid filter example: $filter=contains(namE, 'PATTERN') and
    # name ne 'PATTERN-1'. The following is not a valid filter example:
    # $filter=location eq 'westus'.
    # @param top [Integer] The number of results to return per page for the list
    # operation. Valid range for top parameter is 1 to 100. If not specified, the
    # default number of results to be returned is 20 items per page.
    # @param custom_headers [Hash{String => String}] A hash of custom headers that
    # will be added to the HTTP request.
    #
    # @return [Array<PrivateLinkResource>] operation results.
    #
    def list_by_resource(resource_group_name, parent_type, parent_name, filter:nil, top:nil, custom_headers:nil)
      first_page = list_by_resource_as_lazy(resource_group_name, parent_type, parent_name, filter:filter, top:top, custom_headers:custom_headers)
      first_page.get_all_items
    end

    #
    # List private link resources under specific topic or domain.
    #
    # List all the private link resources under a topic or domain.
    #
    # @param resource_group_name [String] The name of the resource group within the
    # user's subscription.
    # @param parent_type [String] The type of the parent resource. This can be
    # either \'topics\' or \'domains\'.
    # @param parent_name [String] The name of the parent resource (namely, either,
    # the topic name or domain name).
    # @param filter [String] The query used to filter the search results using
    # OData syntax. Filtering is permitted on the 'name' property only and with
    # limited number of OData operations. These operations are: the 'contains'
    # function as well as the following logical operations: not, and, or, eq (for
    # equal), and ne (for not equal). No arithmetic operations are supported. The
    # following is a valid filter example: $filter=contains(namE, 'PATTERN') and
    # name ne 'PATTERN-1'. The following is not a valid filter example:
    # $filter=location eq 'westus'.
    # @param top [Integer] The number of results to return per page for the list
    # operation. Valid range for top parameter is 1 to 100. If not specified, the
    # default number of results to be returned is 20 items per page.
    # @param custom_headers [Hash{String => String}] A hash of custom headers that
    # will be added to the HTTP request.
    #
    # @return [MsRestAzure::AzureOperationResponse] HTTP response information.
    #
    def list_by_resource_with_http_info(resource_group_name, parent_type, parent_name, filter:nil, top:nil, custom_headers:nil)
      list_by_resource_async(resource_group_name, parent_type, parent_name, filter:filter, top:top, custom_headers:custom_headers).value!
    end

    #
    # List private link resources under specific topic or domain.
    #
    # List all the private link resources under a topic or domain.
    #
    # @param resource_group_name [String] The name of the resource group within the
    # user's subscription.
    # @param parent_type [String] The type of the parent resource. This can be
    # either \'topics\' or \'domains\'.
    # @param parent_name [String] The name of the parent resource (namely, either,
    # the topic name or domain name).
    # @param filter [String] The query used to filter the search results using
    # OData syntax. Filtering is permitted on the 'name' property only and with
    # limited number of OData operations. These operations are: the 'contains'
    # function as well as the following logical operations: not, and, or, eq (for
    # equal), and ne (for not equal). No arithmetic operations are supported. The
    # following is a valid filter example: $filter=contains(namE, 'PATTERN') and
    # name ne 'PATTERN-1'. The following is not a valid filter example:
    # $filter=location eq 'westus'.
    # @param top [Integer] The number of results to return per page for the list
    # operation. Valid range for top parameter is 1 to 100. If not specified, the
    # default number of results to be returned is 20 items per page.
    # @param [Hash{String => String}] A hash of custom headers that will be added
    # to the HTTP request.
    #
    # @return [Concurrent::Promise] Promise object which holds the HTTP response.
    #
    def list_by_resource_async(resource_group_name, parent_type, parent_name, filter:nil, top:nil, custom_headers:nil)
      fail ArgumentError, '@client.subscription_id is nil' if @client.subscription_id.nil?
      fail ArgumentError, 'resource_group_name is nil' if resource_group_name.nil?
      fail ArgumentError, 'parent_type is nil' if parent_type.nil?
      fail ArgumentError, 'parent_name is nil' if parent_name.nil?
      fail ArgumentError, '@client.api_version is nil' if @client.api_version.nil?


      request_headers = {}
      request_headers['Content-Type'] = 'application/json; charset=utf-8'

      # Set Headers
      request_headers['x-ms-client-request-id'] = SecureRandom.uuid
      request_headers['accept-language'] = @client.accept_language unless @client.accept_language.nil?
      path_template = 'subscriptions/{subscriptionId}/resourceGroups/{resourceGroupName}/providers/Microsoft.EventGrid/{parentType}/{parentName}/privateLinkResources'

      request_url = @base_url || @client.base_url

      options = {
          middlewares: [[MsRest::RetryPolicyMiddleware, times: 3, retry: 0.02], [:cookie_jar]],
          path_params: {'subscriptionId' => @client.subscription_id,'resourceGroupName' => resource_group_name,'parentType' => parent_type,'parentName' => parent_name},
          query_params: {'api-version' => @client.api_version,'$filter' => filter,'$top' => top},
          headers: request_headers.merge(custom_headers || {}),
          base_url: request_url
      }
      promise = @client.make_request_async(:get, path_template, options)

      promise = promise.then do |result|
        http_response = result.response
        status_code = http_response.status
        response_content = http_response.body
        unless status_code == 200
          error_model = JSON.load(response_content)
          fail MsRestAzure::AzureOperationError.new(result.request, http_response, error_model)
        end

        result.request_id = http_response['x-ms-request-id'] unless http_response['x-ms-request-id'].nil?
        result.correlation_request_id = http_response['x-ms-correlation-request-id'] unless http_response['x-ms-correlation-request-id'].nil?
        result.client_request_id = http_response['x-ms-client-request-id'] unless http_response['x-ms-client-request-id'].nil?
        # Deserialize Response
        if status_code == 200
          begin
            parsed_response = response_content.to_s.empty? ? nil : JSON.load(response_content)
            result_mapper = Azure::EventGrid::Mgmt::V2020_04_01_preview::Models::PrivateLinkResourcesListResult.mapper()
            result.body = @client.deserialize(result_mapper, parsed_response)
          rescue Exception => e
            fail MsRest::DeserializationError.new('Error occurred in deserializing the response', e.message, e.backtrace, result)
          end
        end

        result
      end

      promise.execute
    end

    #
    # List private link resources under specific topic or domain.
    #
    # List all the private link resources under a topic or domain.
    #
    # @param next_page_link [String] The NextLink from the previous successful call
    # to List operation.
    # @param custom_headers [Hash{String => String}] A hash of custom headers that
    # will be added to the HTTP request.
    #
    # @return [PrivateLinkResourcesListResult] operation results.
    #
    def list_by_resource_next(next_page_link, custom_headers:nil)
      response = list_by_resource_next_async(next_page_link, custom_headers:custom_headers).value!
      response.body unless response.nil?
    end

    #
    # List private link resources under specific topic or domain.
    #
    # List all the private link resources under a topic or domain.
    #
    # @param next_page_link [String] The NextLink from the previous successful call
    # to List operation.
    # @param custom_headers [Hash{String => String}] A hash of custom headers that
    # will be added to the HTTP request.
    #
    # @return [MsRestAzure::AzureOperationResponse] HTTP response information.
    #
    def list_by_resource_next_with_http_info(next_page_link, custom_headers:nil)
      list_by_resource_next_async(next_page_link, custom_headers:custom_headers).value!
    end

    #
    # List private link resources under specific topic or domain.
    #
    # List all the private link resources under a topic or domain.
    #
    # @param next_page_link [String] The NextLink from the previous successful call
    # to List operation.
    # @param [Hash{String => String}] A hash of custom headers that will be added
    # to the HTTP request.
    #
    # @return [Concurrent::Promise] Promise object which holds the HTTP response.
    #
    def list_by_resource_next_async(next_page_link, custom_headers:nil)
      fail ArgumentError, 'next_page_link is nil' if next_page_link.nil?


      request_headers = {}
      request_headers['Content-Type'] = 'application/json; charset=utf-8'

      # Set Headers
      request_headers['x-ms-client-request-id'] = SecureRandom.uuid
      request_headers['accept-language'] = @client.accept_language unless @client.accept_language.nil?
      path_template = '{nextLink}'

      request_url = @base_url || @client.base_url

      options = {
          middlewares: [[MsRest::RetryPolicyMiddleware, times: 3, retry: 0.02], [:cookie_jar]],
          skip_encoding_path_params: {'nextLink' => next_page_link},
          headers: request_headers.merge(custom_headers || {}),
          base_url: request_url
      }
      promise = @client.make_request_async(:get, path_template, options)

      promise = promise.then do |result|
        http_response = result.response
        status_code = http_response.status
        response_content = http_response.body
        unless status_code == 200
          error_model = JSON.load(response_content)
          fail MsRestAzure::AzureOperationError.new(result.request, http_response, error_model)
        end

        result.request_id = http_response['x-ms-request-id'] unless http_response['x-ms-request-id'].nil?
        result.correlation_request_id = http_response['x-ms-correlation-request-id'] unless http_response['x-ms-correlation-request-id'].nil?
        result.client_request_id = http_response['x-ms-client-request-id'] unless http_response['x-ms-client-request-id'].nil?
        # Deserialize Response
        if status_code == 200
          begin
            parsed_response = response_content.to_s.empty? ? nil : JSON.load(response_content)
            result_mapper = Azure::EventGrid::Mgmt::V2020_04_01_preview::Models::PrivateLinkResourcesListResult.mapper()
            result.body = @client.deserialize(result_mapper, parsed_response)
          rescue Exception => e
            fail MsRest::DeserializationError.new('Error occurred in deserializing the response', e.message, e.backtrace, result)
          end
        end

        result
      end

      promise.execute
    end

    #
    # List private link resources under specific topic or domain.
    #
    # List all the private link resources under a topic or domain.
    #
    # @param resource_group_name [String] The name of the resource group within the
    # user's subscription.
    # @param parent_type [String] The type of the parent resource. This can be
    # either \'topics\' or \'domains\'.
    # @param parent_name [String] The name of the parent resource (namely, either,
    # the topic name or domain name).
    # @param filter [String] The query used to filter the search results using
    # OData syntax. Filtering is permitted on the 'name' property only and with
    # limited number of OData operations. These operations are: the 'contains'
    # function as well as the following logical operations: not, and, or, eq (for
    # equal), and ne (for not equal). No arithmetic operations are supported. The
    # following is a valid filter example: $filter=contains(namE, 'PATTERN') and
    # name ne 'PATTERN-1'. The following is not a valid filter example:
    # $filter=location eq 'westus'.
    # @param top [Integer] The number of results to return per page for the list
    # operation. Valid range for top parameter is 1 to 100. If not specified, the
    # default number of results to be returned is 20 items per page.
    # @param custom_headers [Hash{String => String}] A hash of custom headers that
    # will be added to the HTTP request.
    #
    # @return [PrivateLinkResourcesListResult] which provide lazy access to pages
    # of the response.
    #
    def list_by_resource_as_lazy(resource_group_name, parent_type, parent_name, filter:nil, top:nil, custom_headers:nil)
      response = list_by_resource_async(resource_group_name, parent_type, parent_name, filter:filter, top:top, custom_headers:custom_headers).value!
      unless response.nil?
        page = response.body
        page.next_method = Proc.new do |next_page_link|
          list_by_resource_next_async(next_page_link, custom_headers:custom_headers)
        end
        page
      end
    end

  end
end
